/* Copyright (C) 2007 P.L. Lucas
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA. 
 */

#include "command_list.h"
#include "projects/projects.h"
#include <QVBoxLayout>
#include <QListWidget>
#include <QApplication>
#include <QClipboard>

CommandList::CommandList( QWidget * parent ):BaseWidget(parent)
{
	setWindowTitle("Commands' List");
	setWindowIcon(QIcon(QString(ICON_PATH)+"/news_section.png"));
	
	QVBoxLayout *layout=new QVBoxLayout;
	
	list=new QListView(this);
	model=new StringListModel();
	list->setModel(model);
	list->setSelectionMode(QAbstractItemView::ExtendedSelection);
	list->show();
	
	QObject::connect(list, SIGNAL(activated(const QModelIndex & )), this, SLOT(itemClicked (const QModelIndex & )) );
	
	layout->addWidget(list);
	
	setLayout(layout);
	
	list->setContextMenuPolicy(Qt::CustomContextMenu);
	
	context_menu=new QMenu(this);
	QAction *copy_clipboard=new QAction(tr("Copy"), this);
	connect(copy_clipboard, SIGNAL(triggered()), this, SLOT(copy_clipboard_callback()) );
	
	context_menu->addAction(copy_clipboard);
	
	connect(list, SIGNAL( customContextMenuRequested ( const QPoint & )  ), this, SLOT( contextMenu_cb( const QPoint & ) ) );
}

CommandList::~CommandList()
{
	QStringList history=model->stringList();
	Projects::saveListCommandHistory(session->getProjectName(), history);
}

void CommandList::setLineEdit(QLineEdit *lineEdit)
{
	this->lineEdit=lineEdit;
}

void CommandList::set_list(QStringList list)
{
	QStringList history;
	history << project_history << list;
	while(history.size()>127)
	{
		history.removeFirst();
		project_history.removeFirst();
	}
	model->setStringList(history);
	QModelIndex item=model->index(history.size()-1);
	this->list->scrollTo(item);
}

void CommandList::itemClicked (const QModelIndex & item )
{
	lineEdit->setText(item.data().toString());
}

void CommandList::setSession(Session *session)
{
	this->session=session;
	
	QStringList history=Projects::listCommandHistory( session->getProjectName() );
	
	project_history.clear();
	project_history << history;
	model->setStringList(history);
	//this->set_list(history);
}

void CommandList::copy_clipboard_callback()
{
	QClipboard *clipboard=QApplication::clipboard();
	QString history;
	for(int i=0;i<model->rowCount();i++)
	{
		QModelIndex item=model->index(i);
		if( list->selectionModel()->isSelected(item) )
			history += item.data().toString()+"\n";
	}
	clipboard->setText(history);
}

void CommandList::contextMenu_cb( const QPoint & /*pos*/)
{
	context_menu->popup(QCursor::pos () );
}


StringListModel::StringListModel():QStringListModel()
{
}

Qt::ItemFlags StringListModel::flags(const QModelIndex &index) const
{
	if (!index.isValid())
         return Qt::ItemIsEnabled;

     return QAbstractItemModel::flags(index);
}
